@@ -37,14 +37,6 @@ all_errors = ValueError
 try:
     import keyring
     HAS_KEYRING = True
-    try:
-        if isinstance(keyring.get_keyring(), keyring.backend.GnomeKeyring):
-            import gnomekeyring
-            all_errors = (ValueError,
-                          gnomekeyring.IOError,
-                          gnomekeyring.NoKeyringDaemonError)
-    except Exception:
-        pass
 except ImportError:
     pass
 
@@ -53,6 +45,8 @@ import novaclient.auth_plugin
 from novaclient import client
 from novaclient import exceptions as exc
 import novaclient.extension
+from novaclient.openstack.common import cliutils
+from novaclient.openstack.common.gettextutils import _
 from novaclient.openstack.common import strutils
 from novaclient import utils
 from novaclient.v1_1 import shell as shell_v1_1
@@ -60,7 +54,13 @@ from novaclient.v3 import shell as shell_v3
 
 DEFAULT_OS_COMPUTE_API_VERSION = "1.1"
 DEFAULT_NOVA_ENDPOINT_TYPE = 'publicURL'
-DEFAULT_NOVA_SERVICE_TYPE = 'compute'
+# NOTE(cyeoh): Having the service type dependent on the API version
+# is pretty ugly, but we have to do this because traditionally the
+# catalog entry for compute points directly to the V2 API rather than
+# the root, and then doing version discovery.
+DEFAULT_NOVA_SERVICE_TYPE_MAP = {'1.1': 'compute',
+                                 '2': 'compute',
+                                 '3': 'computev3'}
 
 logger = logging.getLogger(__name__)
 
@@ -71,10 +71,10 @@ def positive_non_zero_float(text):
     try:
         value = float(text)
     except ValueError:
-        msg = "%s must be a float" % text
+        msg = _("%s must be a float") % text
         raise argparse.ArgumentTypeError(msg)
     if value <= 0:
-        msg = "%s must be greater than 0" % text
+        msg = _("%s must be greater than 0") % text
         raise argparse.ArgumentTypeError(msg)
     return value
 
@@ -84,6 +84,7 @@ class SecretsHelper(object):
         self.args = args
         self.client = client
         self.key = None
+        self._password = None
 
     def _validate_string(self, text):
         if text is None or len(text) == 0:
@@ -137,7 +138,8 @@ class SecretsHelper(object):
             # Nothing changed....
             return
         if not all([management_url, auth_token, tenant_id]):
-            raise ValueError("Unable to save empty management url/auth token")
+            raise ValueError(_("Unable to save empty management url/auth "
+                               "token"))
         value = "|".join([str(auth_token),
                           str(management_url),
                           str(tenant_id)])
@@ -145,10 +147,21 @@ class SecretsHelper(object):
 
     @property
     def password(self):
-        if self._validate_string(self.args.os_password):
-            return self.args.os_password
-        verify_pass = utils.bool_from_str(utils.env("OS_VERIFY_PASSWORD"))
-        return self._prompt_password(verify_pass)
+        # Cache password so we prompt user at most once
+        if self._password:
+            pass
+        elif self._validate_string(self.args.os_password):
+            self._password = self.args.os_password
+        else:
+            verify_pass = strutils.bool_from_string(
+                utils.env("OS_VERIFY_PASSWORD", default=False), True)
+            self._password = self._prompt_password(verify_pass)
+        if not self._password:
+            raise exc.CommandError(
+                'Expecting a password provided via either '
+                '--os-password, env[OS_PASSWORD], or '
+                'prompted response')
+        return self._password
 
     @property
     def management_url(self):
@@ -209,8 +222,8 @@ class NovaClientArgumentParser(argparse.ArgumentParser):
         #FIXME(lzyeval): if changes occur in argparse.ArgParser._check_value
         choose_from = ' (choose from'
         progparts = self.prog.partition(' ')
-        self.exit(2, "error: %(errmsg)s\nTry '%(mainp)s help %(subp)s'"
-                     " for more information.\n" %
+        self.exit(2, _("error: %(errmsg)s\nTry '%(mainp)s help %(subp)s'"
+                     " for more information.\n") %
                      {'errmsg': message.split(choose_from)[0],
                       'mainp': progparts[0],
                       'subp': progparts[2]})
@@ -241,72 +254,67 @@ class OpenStackComputeShell(object):
         parser.add_argument('--debug',
             default=False,
             action='store_true',
-            help="Print debugging output")
-
-        parser.add_argument('--no-cache',
-            default=not utils.bool_from_str(
-                    utils.env('OS_NO_CACHE', default='true')),
-            action='store_false',
-            dest='os_cache',
-            help=argparse.SUPPRESS)
-        parser.add_argument('--no_cache',
-            action='store_false',
-            dest='os_cache',
-            help=argparse.SUPPRESS)
+            help=_("Print debugging output"))
 
         parser.add_argument('--os-cache',
-            default=utils.env('OS_CACHE', default=False),
+            default=strutils.bool_from_string(
+                utils.env('OS_CACHE', default=False), True),
             action='store_true',
-            help="Use the auth token cache.")
+            help=_("Use the auth token cache. Defaults to False if "
+                   "env[OS_CACHE] is not set."))
 
         parser.add_argument('--timings',
             default=False,
             action='store_true',
-            help="Print call timing info")
+            help=_("Print call timing info"))
 
         parser.add_argument('--timeout',
             default=600,
             metavar='<seconds>',
             type=positive_non_zero_float,
-            help="Set HTTP call timeout (in seconds)")
+            help=_("Set HTTP call timeout (in seconds)"))
+
+        parser.add_argument('--os-auth-token',
+                default=utils.env('OS_AUTH_TOKEN'),
+                help='Defaults to env[OS_AUTH_TOKEN]')
 
         parser.add_argument('--os-username',
             metavar='<auth-user-name>',
             default=utils.env('OS_USERNAME', 'NOVA_USERNAME'),
-            help='Defaults to env[OS_USERNAME].')
+            help=_('Defaults to env[OS_USERNAME].'))
         parser.add_argument('--os_username',
             help=argparse.SUPPRESS)
 
         parser.add_argument('--os-password',
             metavar='<auth-password>',
             default=utils.env('OS_PASSWORD', 'NOVA_PASSWORD'),
-            help='Defaults to env[OS_PASSWORD].')
+            help=_('Defaults to env[OS_PASSWORD].'))
         parser.add_argument('--os_password',
             help=argparse.SUPPRESS)
 
         parser.add_argument('--os-tenant-name',
             metavar='<auth-tenant-name>',
             default=utils.env('OS_TENANT_NAME', 'NOVA_PROJECT_ID'),
-            help='Defaults to env[OS_TENANT_NAME].')
+            help=_('Defaults to env[OS_TENANT_NAME].'))
         parser.add_argument('--os_tenant_name',
             help=argparse.SUPPRESS)
 
         parser.add_argument('--os-tenant-id',
             metavar='<auth-tenant-id>',
             default=utils.env('OS_TENANT_ID'),
-            help='Defaults to env[OS_TENANT_ID].')
+            help=_('Defaults to env[OS_TENANT_ID].'))
 
         parser.add_argument('--os-auth-url',
             metavar='<auth-url>',
             default=utils.env('OS_AUTH_URL', 'NOVA_URL'),
-            help='Defaults to env[OS_AUTH_URL].')
+            help=_('Defaults to env[OS_AUTH_URL].'))
         parser.add_argument('--os_auth_url',
             help=argparse.SUPPRESS)
 
         parser.add_argument('--os-region-name',
             metavar='<region-name>',
             default=utils.env('OS_REGION_NAME', 'NOVA_REGION_NAME'),
-            help='Defaults to env[OS_REGION_NAME].')
+            help=_('Defaults to env[OS_REGION_NAME].'))
         parser.add_argument('--os_region_name',
             help=argparse.SUPPRESS)
 
@@ -319,21 +327,21 @@ class OpenStackComputeShell(object):
 
         parser.add_argument('--service-type',
             metavar='<service-type>',
-            help='Defaults to compute for most actions')
+            help=_('Defaults to compute for most actions'))
         parser.add_argument('--service_type',
             help=argparse.SUPPRESS)
 
         parser.add_argument('--service-name',
             metavar='<service-name>',
             default=utils.env('NOVA_SERVICE_NAME'),
-            help='Defaults to env[NOVA_SERVICE_NAME]')
+            help=_('Defaults to env[NOVA_SERVICE_NAME]'))
         parser.add_argument('--service_name',
             help=argparse.SUPPRESS)
 
         parser.add_argument('--volume-service-name',
             metavar='<volume-service-name>',
             default=utils.env('NOVA_VOLUME_SERVICE_NAME'),
-            help='Defaults to env[NOVA_VOLUME_SERVICE_NAME]')
+            help=_('Defaults to env[NOVA_VOLUME_SERVICE_NAME]'))
         parser.add_argument('--volume_service_name',
             help=argparse.SUPPRESS)
 
@@ -341,7 +349,7 @@ class OpenStackComputeShell(object):
             metavar='<endpoint-type>',
             default=utils.env('NOVA_ENDPOINT_TYPE',
                         default=DEFAULT_NOVA_ENDPOINT_TYPE),
-            help='Defaults to env[NOVA_ENDPOINT_TYPE] or '
+            help=_('Defaults to env[NOVA_ENDPOINT_TYPE] or ')
                     + DEFAULT_NOVA_ENDPOINT_TYPE + '.')
         # NOTE(dtroyer): We can't add --endpoint_type here due to argparse
         #                thinking usage-list --end is ambiguous; but it
@@ -354,8 +362,8 @@ class OpenStackComputeShell(object):
             metavar='<compute-api-ver>',
             default=utils.env('OS_COMPUTE_API_VERSION',
                 default=DEFAULT_OS_COMPUTE_API_VERSION),
-            help='Accepts 1.1 or 3, '
-                 'defaults to env[OS_COMPUTE_API_VERSION].')
+            help=_('Accepts 1.1 or 3, '
+                 'defaults to env[OS_COMPUTE_API_VERSION].'))
         parser.add_argument('--os_compute_api_version',
             help=argparse.SUPPRESS)
 
@@ -369,10 +377,10 @@ class OpenStackComputeShell(object):
         parser.add_argument('--insecure',
             default=utils.env('NOVACLIENT_INSECURE', default=False),
             action='store_true',
-            help="Explicitly allow novaclient to perform \"insecure\" "
+            help=_("Explicitly allow novaclient to perform \"insecure\" "
                  "SSL (https) requests. The server's certificate will "
                  "not be verified against any certificate authorities. "
-                 "This option should be used with caution.")
+                 "This option should be used with caution."))
 
         parser.add_argument('--bypass-url',
             metavar='<bypass-url>',
@@ -468,7 +476,7 @@ class OpenStackComputeShell(object):
 
     def _find_actions(self, subparsers, actions_module):
         for attr in (a for a in dir(actions_module) if a.startswith('do_')):
-            # I prefer to be hypen-separated instead of underscores.
+            # I prefer to be hyphen-separated instead of underscores.
             command = attr[3:].replace('_', '-')
             callback = getattr(actions_module, attr)
             desc = callback.__doc__ or ''
@@ -501,7 +509,6 @@ class OpenStackComputeShell(object):
                             format=streamformat)
 
     def main(self, argv):
-
         # Parse args once to find version and debug settings
         parser = self.get_base_parser()
         (options, args) = parser.parse_known_args(argv)
@@ -542,73 +549,95 @@ class OpenStackComputeShell(object):
             self.do_bash_completion(args)
             return 0
 
-        (os_username, os_tenant_name, os_tenant_id, os_auth_url,
-                os_region_name, os_auth_system, endpoint_type, insecure,
-                service_type, service_name, volume_service_name,
-                bypass_url, os_cache, cacert, timeout) = (
-                        args.os_username,
-                        args.os_tenant_name, args.os_tenant_id,
-                        args.os_auth_url,
-                        args.os_region_name, args.os_auth_system,
-                        args.endpoint_type, args.insecure, args.service_type,
-                        args.service_name, args.volume_service_name,
-                        args.bypass_url, args.os_cache,
-                        args.os_cacert, args.timeout)
+        os_username = args.os_username
+        os_password = None  # Fetched and set later as needed
+        os_tenant_name = args.os_tenant_name
+        os_tenant_id = args.os_tenant_id
+        os_auth_url = args.os_auth_url
+        os_region_name = args.os_region_name
+        os_auth_system = args.os_auth_system
+        endpoint_type = args.endpoint_type
+        insecure = args.insecure
+        service_type = args.service_type
+        service_name = args.service_name
+        volume_service_name = args.volume_service_name
+        bypass_url = args.bypass_url
+        os_cache = args.os_cache
+        cacert = args.os_cacert
+        timeout = args.timeout
+
+        # We may have either, both or none of these.
+        # If we have both, we don't need USERNAME, PASSWORD etc.
+        # Fill in the blanks from the SecretsHelper if possible.
+        # Finally, authenticate unless we have both.
+        # Note if we don't auth we probably don't have a tenant ID so we can't
+        # cache the token.
+        auth_token = args.os_auth_token if args.os_auth_token else None
+        management_url = bypass_url if bypass_url else None
 
         if os_auth_system and os_auth_system != "keystone":
             auth_plugin = novaclient.auth_plugin.load_plugin(os_auth_system)
         else:
             auth_plugin = None
 
-        # Fetched and set later as needed
-        os_password = None
-
         if not endpoint_type:
             endpoint_type = DEFAULT_NOVA_ENDPOINT_TYPE
 
         if not service_type:
-            service_type = DEFAULT_NOVA_SERVICE_TYPE
+            os_compute_api_version = (options.os_compute_api_version or
+                                      DEFAULT_OS_COMPUTE_API_VERSION)
+            try:
+                service_type = DEFAULT_NOVA_SERVICE_TYPE_MAP[
+                    os_compute_api_version]
+            except KeyError:
+                service_type = DEFAULT_NOVA_SERVICE_TYPE_MAP[
+                    DEFAULT_OS_COMPUTE_API_VERSION]
             service_type = utils.get_service_type(args.func) or service_type
 
+        # If we have an auth token but no management_url, we must auth anyway.
+        # Expired tokens are handled by client.py:_cs_request
+        must_auth = not (cliutils.isunauthenticated(args.func)
+                         or (auth_token and management_url))
+
         #FIXME(usrleon): Here should be restrict for project id same as
         # for os_username or os_password but for compatibility it is not.
-        if not utils.isunauthenticated(args.func):
+        if must_auth:
             if auth_plugin:
                 auth_plugin.parse_opts(args)
 
             if not auth_plugin or not auth_plugin.opts:
                 if not os_username:
-                    raise exc.CommandError("You must provide a username "
-                            "via either --os-username or env[OS_USERNAME]")
+                    raise exc.CommandError(_("You must provide a username "
+                            "via either --os-username or env[OS_USERNAME]"))
 
             if not os_tenant_name and not os_tenant_id:
-                raise exc.CommandError("You must provide a tenant name "
+                raise exc.CommandError(_("You must provide a tenant name "
                         "or tenant id via --os-tenant-name, "
                         "--os-tenant-id, env[OS_TENANT_NAME] "
-                        "or env[OS_TENANT_ID]")
+                        "or env[OS_TENANT_ID]"))
 
             if not os_auth_url:
                 if os_auth_system and os_auth_system != 'keystone':
                     os_auth_url = auth_plugin.get_auth_url()
 
             if not os_auth_url:
-                    raise exc.CommandError("You must provide an auth url "
+                    raise exc.CommandError(_("You must provide an auth url "
                             "via either --os-auth-url or env[OS_AUTH_URL] "
                             "or specify an auth_system which defines a "
                             "default url with --os-auth-system "
-                            "or env[OS_AUTH_SYSTEM]")
+                            "or env[OS_AUTH_SYSTEM]"))
 
         if (options.os_compute_api_version and
                 options.os_compute_api_version != '1.0'):
             if not os_tenant_name and not os_tenant_id:
-                raise exc.CommandError("You must provide a tenant name "
+                raise exc.CommandError(_("You must provide a tenant name "
                         "or tenant id via --os-tenant-name, "
                         "--os-tenant-id, env[OS_TENANT_NAME] "
-                        "or env[OS_TENANT_ID]")
+                        "or env[OS_TENANT_ID]"))
 
             if not os_auth_url:
-                raise exc.CommandError("You must provide an auth url "
-                        "via either --os-auth-url or env[OS_AUTH_URL]")
+                raise exc.CommandError(_("You must provide an auth url "
+                        "via either --os-auth-url or env[OS_AUTH_URL]"))
 
         self.cs = client.Client(options.os_compute_api_version, os_username,
                 os_password, os_tenant_name, tenant_id=os_tenant_id,
@@ -616,7 +645,7 @@ class OpenStackComputeShell(object):
                 region_name=os_region_name, endpoint_type=endpoint_type,
                 extensions=self.extensions, service_type=service_type,
                 service_name=service_name, auth_system=os_auth_system,
-                auth_plugin=auth_plugin,
+                auth_plugin=auth_plugin, auth_token=auth_token,
                 volume_service_name=volume_service_name,
                 timings=args.timings, bypass_url=bypass_url,
                 os_cache=os_cache, http_log_debug=options.debug,
@@ -624,7 +653,7 @@ class OpenStackComputeShell(object):
 
         # Now check for the password/token of which pieces of the
         # identifying keyring key can come from the underlying client
-        if not utils.isunauthenticated(args.func):
+        if must_auth:
             helper = SecretsHelper(args, self.cs.client)
             if (auth_plugin and auth_plugin.opts and
                     "os_password" not in auth_plugin.opts):
@@ -632,43 +661,51 @@ class OpenStackComputeShell(object):
             else:
                 use_pw = True
 
-            tenant_id, auth_token, management_url = (helper.tenant_id,
-                                                     helper.auth_token,
-                                                     helper.management_url)
+            tenant_id = helper.tenant_id
+            # Allow commandline to override cache
+            if not auth_token:
+                auth_token = helper.auth_token
+            if not management_url:
+                management_url = helper.management_url
             if tenant_id and auth_token and management_url:
                 self.cs.client.tenant_id = tenant_id
                 self.cs.client.auth_token = auth_token
                 self.cs.client.management_url = management_url
-                # Try to auth with the given info, if it fails
-                # go into password mode...
-                try:
-                    self.cs.authenticate()
-                    use_pw = False
-                except (exc.Unauthorized, exc.AuthorizationFailure):
-                    # Likely it expired or just didn't work...
-                    self.cs.client.auth_token = None
-                    self.cs.client.management_url = None
-            if use_pw:
-                # Auth using token must have failed or not happened
-                # at all, so now switch to password mode and save
-                # the token when its gotten... using our keyring
-                # saver
-                os_password = helper.password
-                if not os_password:
-                    raise exc.CommandError(
-                        'Expecting a password provided via either '
-                        '--os-password, env[OS_PASSWORD], or '
-                        'prompted response')
-                self.cs.client.password = os_password
+                self.cs.client.password_func = lambda: helper.password
+            elif use_pw:
+                # We're missing something, so auth with user/pass and save
+                # the result in our helper.
+                self.cs.client.password = helper.password
                 self.cs.client.keyring_saver = helper
 
         try:
-            if not utils.isunauthenticated(args.func):
+            # This does a couple of bits which are useful even if we've
+            # got the token + service URL already. It exits fast in that case.
+            if not cliutils.isunauthenticated(args.func):
                 self.cs.authenticate()
         except exc.Unauthorized:
-            raise exc.CommandError("Invalid OpenStack Nova credentials.")
+            raise exc.CommandError(_("Invalid OpenStack Nova credentials."))
         except exc.AuthorizationFailure:
-            raise exc.CommandError("Unable to authorize user")
+            raise exc.CommandError(_("Unable to authorize user"))
+
+        if os_compute_api_version == "3" and service_type != 'image':
+            # NOTE(cyeoh): create an image based client because the
+            # images api is no longer proxied by the V3 API and we
+            # sometimes need to be able to look up images information
+            # via glance when connected to the nova api.
+            image_service_type = 'image'
+            self.cs.image_cs = client.Client(
+                options.os_compute_api_version, os_username,
+                os_password, os_tenant_name, tenant_id=os_tenant_id,
+                auth_url=os_auth_url, insecure=insecure,
+                region_name=os_region_name, endpoint_type=endpoint_type,
+                extensions=self.extensions, service_type=image_service_type,
+                service_name=service_name, auth_system=os_auth_system,
+                auth_plugin=auth_plugin,
+                volume_service_name=volume_service_name,
+                timings=args.timings, bypass_url=bypass_url,
+                os_cache=os_cache, http_log_debug=options.debug,
+                cacert=cacert, timeout=timeout)
 
         args.func(self.cs, args)
 
@@ -718,7 +755,7 @@ class OpenStackComputeShell(object):
             if args.command in self.subcommands:
                 self.subcommands[args.command].print_help()
             else:
-                raise exc.CommandError("'%s' is not a valid subcommand" %
+                raise exc.CommandError(_("'%s' is not a valid subcommand") %
                                        args.command)
         else:
             self.parser.print_help()
@@ -741,6 +778,9 @@ def main():
         print("ERROR: %s" % strutils.safe_encode(six.text_type(e)),
               file=sys.stderr)
         sys.exit(1)
+    except KeyboardInterrupt as e:
+        print("Shutting down novaclient", file=sys.stderr)
+        sys.exit(1)
 
 
 if __name__ == "__main__":
