@@ -20,9 +20,11 @@ import requests
 import novaclient.client
 import novaclient.extension
 import novaclient.tests.fakes as fakes
+from novaclient.tests import utils
 import novaclient.v1_1.client
 import novaclient.v3.client
-from novaclient.tests import utils
+
+import json
 
 
 class ClientTest(utils.TestCase):
@@ -71,8 +73,16 @@ class ClientTest(utils.TestCase):
             reauth_headers = {'Content-Type': 'application/json',
                               'Accept': 'application/json',
                               'User-Agent': 'python-novaclient'}
-            data = ('{"auth": {"tenantName": "project", "passwordCredentials":'
-                    ' {"username": "user", "password": "password"}}}')
+            data = {
+                "auth": {
+                    "tenantName": "project",
+                    "passwordCredentials": {
+                        "username": "user",
+                        "password": "password"
+                        }
+                }
+            }
+
             expected = [mock.call('GET',
                                   'http://example.com/servers/detail',
                                   timeout=mock.ANY,
@@ -82,7 +92,7 @@ class ClientTest(utils.TestCase):
                                   timeout=mock.ANY,
                                   headers=reauth_headers,
                                   allow_redirects=mock.ANY,
-                                  data=data,
+                                  data=json.dumps(data),
                                   verify=mock.ANY)]
             self.assertEqual(mock_request.call_args_list, expected)
 
@@ -161,7 +171,7 @@ class ClientTest(utils.TestCase):
         self.assertEqual(["somevalue"], cs.get_timings())
 
         cs.reset_timings()
-        self.assertEquals(0, len(cs.get_timings()))
+        self.assertEqual(0, len(cs.get_timings()))
 
     def test_clent_extensions_v3(self):
         fake_attribute_name1 = "FakeAttribute1"
@@ -176,8 +186,8 @@ class ClientTest(utils.TestCase):
         cs = novaclient.v3.client.Client("user", "password", "project_id",
                                            auth_url="foo/v2",
                                            extensions=extensions)
-        self.assertTrue(isinstance(getattr(cs, fake_attribute_name1, None),
-                                   fakes.FakeManager))
+        self.assertIsInstance(getattr(cs, fake_attribute_name1, None),
+                              fakes.FakeManager)
         self.assertFalse(hasattr(cs, fake_attribute_name2))
 
     @mock.patch.object(novaclient.client.HTTPClient, 'authenticate')
@@ -186,3 +196,23 @@ class ClientTest(utils.TestCase):
                                            auth_url="foo/v2")
         cs.authenticate()
         self.assertTrue(mock_authenticate.called)
+
+    def test_get_password_simple(self):
+        cs = novaclient.client.HTTPClient("user", "password", "", "")
+        cs.password_func = mock.Mock()
+        self.assertEqual(cs._get_password(), "password")
+        self.assertFalse(cs.password_func.called)
+
+    def test_get_password_none(self):
+        cs = novaclient.client.HTTPClient("user", None, "", "")
+        self.assertIsNone(cs._get_password())
+
+    def test_get_password_func(self):
+        cs = novaclient.client.HTTPClient("user", None, "", "")
+        cs.password_func = mock.Mock(return_value="password")
+        self.assertEqual(cs._get_password(), "password")
+        cs.password_func.assert_called_once_with()
+
+        cs.password_func = mock.Mock()
+        self.assertEqual(cs._get_password(), "password")
+        self.assertFalse(cs.password_func.called)
