@@ -1,6 +1,19 @@
-import io
+#
+#    Licensed under the Apache License, Version 2.0 (the "License"); you may
+#    not use this file except in compliance with the License. You may obtain
+#    a copy of the License at
+#
+#         http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#    License for the specific language governing permissions and limitations
+#    under the License.
+
 import prettytable
 import re
+import six
 import sys
 
 from distutils.version import StrictVersion
@@ -36,15 +49,16 @@ class ShellTest(utils.TestCase):
         self.useFixture(fixtures.MonkeyPatch(
                         'novaclient.client.get_client_class',
                         mock.MagicMock))
-        self.nc_util = mock.patch('novaclient.utils.isunauthenticated').start()
+        self.nc_util = mock.patch(
+            'novaclient.openstack.common.cliutils.isunauthenticated').start()
         self.nc_util.return_value = False
 
     def shell(self, argstr, exitcodes=(0,)):
         orig = sys.stdout
         orig_stderr = sys.stderr
         try:
-            sys.stdout = io.BytesIO()
-            sys.stderr = io.BytesIO()
+            sys.stdout = six.StringIO()
+            sys.stderr = six.StringIO()
             _shell = novaclient.shell.OpenStackComputeShell()
             _shell.main(argstr.split())
         except SystemExit:
@@ -169,6 +183,8 @@ class ShellTest(utils.TestCase):
     @mock.patch('sys.stdin', side_effect=mock.MagicMock)
     @mock.patch('getpass.getpass', return_value='password')
     def test_password(self, mock_getpass, mock_stdin):
+        mock_stdin.encoding = "utf-8"
+
         # default output of empty tables differs depending between prettytable
         # versions
         if (hasattr(prettytable, '__version__') and
@@ -198,3 +214,33 @@ class ShellTest(utils.TestCase):
             self.assertEqual(required, message.args)
         else:
             self.fail('CommandError not raised')
+
+    def _test_service_type(self, version, service_type, mock_client):
+        if version is None:
+            cmd = 'list'
+        else:
+            cmd = '--os-compute-api-version %s list' % version
+        self.make_env()
+        self.shell(cmd)
+        _, client_kwargs = mock_client.call_args_list[0]
+        self.assertEqual(service_type, client_kwargs['service_type'])
+
+    @mock.patch('novaclient.client.Client')
+    def test_default_service_type(self, mock_client):
+        self._test_service_type(None, 'compute', mock_client)
+
+    @mock.patch('novaclient.client.Client')
+    def test_v1_1_service_type(self, mock_client):
+        self._test_service_type('1.1', 'compute', mock_client)
+
+    @mock.patch('novaclient.client.Client')
+    def test_v2_service_type(self, mock_client):
+        self._test_service_type('2', 'compute', mock_client)
+
+    @mock.patch('novaclient.client.Client')
+    def test_v3_service_type(self, mock_client):
+        self._test_service_type('3', 'computev3', mock_client)
+
+    @mock.patch('novaclient.client.Client')
+    def test_v_unknown_service_type(self, mock_client):
+        self._test_service_type('unknown', 'compute', mock_client)
