@@ -1,3 +1,16 @@
+#
+#    Licensed under the Apache License, Version 2.0 (the "License"); you may
+#    not use this file except in compliance with the License. You may obtain
+#    a copy of the License at
+#
+#         http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#    License for the specific language governing permissions and limitations
+#    under the License.
+
 import os
 
 import fixtures
@@ -29,13 +42,13 @@ class TestResponse(requests.Response):
     """
 
     def __init__(self, data):
+        super(TestResponse, self).__init__()
         self._text = None
-        super(TestResponse, self)
         if isinstance(data, dict):
-            self.status_code = data.get('status_code', None)
-            self.headers = data.get('headers', None)
+            self.status_code = data.get('status_code')
+            self.headers = data.get('headers')
             # Fake the text attribute to streamline Response creation
-            self._text = data.get('text', None)
+            self._text = data.get('text')
         else:
             self.status_code = data
 
