@@ -14,9 +14,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from novaclient.v1_1 import floating_ip_pools
 from novaclient.tests import utils
 from novaclient.tests.v1_1 import fakes
+from novaclient.v1_1 import floating_ip_pools
 
 
 cs = fakes.FakeClient()
@@ -27,5 +27,5 @@ class TestFloatingIPPools(utils.TestCase):
     def test_list_floating_ips(self):
         fl = cs.floating_ip_pools.list()
         cs.assert_called('GET', '/os-floating-ip-pools')
-        [self.assertTrue(isinstance(f, floating_ip_pools.FloatingIPPool))
+        [self.assertIsInstance(f, floating_ip_pools.FloatingIPPool)
                                                                    for f in fl]
