@@ -29,14 +29,14 @@ class QuotaClassSetsTest(utils.TestCase):
 
     def test_update_quota(self):
         q = cs.quota_classes.get('test')
-        q.update(volumes=2)
+        q.update(cores=2)
         cs.assert_called('PUT', '/os-quota-class-sets/test')
 
     def test_refresh_quota(self):
         q = cs.quota_classes.get('test')
         q2 = cs.quota_classes.get('test')
-        self.assertEqual(q.volumes, q2.volumes)
-        q2.volumes = 0
-        self.assertNotEqual(q.volumes, q2.volumes)
+        self.assertEqual(q.cores, q2.cores)
+        q2.cores = 0
+        self.assertNotEqual(q.cores, q2.cores)
         q2.get()
-        self.assertEqual(q.volumes, q2.volumes)
+        self.assertEqual(q.cores, q2.cores)
