@@ -1,12 +1,24 @@
 # -*- coding: utf-8 -*-
+#
+#    Licensed under the Apache License, Version 2.0 (the "License"); you may
+#    not use this file except in compliance with the License. You may obtain
+#    a copy of the License at
+#
+#         http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#    License for the specific language governing permissions and limitations
+#    under the License.
 
 import mock
 import six
 
 from novaclient import exceptions
-from novaclient.v1_1 import servers
 from novaclient.tests import utils
 from novaclient.tests.v1_1 import fakes
+from novaclient.v1_1 import servers
 
 
 cs = fakes.FakeClient()
@@ -17,30 +29,30 @@ class ServersTest(utils.TestCase):
     def test_list_servers(self):
         sl = cs.servers.list()
         cs.assert_called('GET', '/servers/detail')
-        [self.assertTrue(isinstance(s, servers.Server)) for s in sl]
+        [self.assertIsInstance(s, servers.Server) for s in sl]
 
     def test_list_servers_undetailed(self):
         sl = cs.servers.list(detailed=False)
         cs.assert_called('GET', '/servers')
-        [self.assertTrue(isinstance(s, servers.Server)) for s in sl]
+        [self.assertIsInstance(s, servers.Server) for s in sl]
 
     def test_list_servers_with_marker_limit(self):
         sl = cs.servers.list(marker=1234, limit=2)
-        cs.assert_called('GET', '/servers/detail?marker=1234&limit=2')
+        cs.assert_called('GET', '/servers/detail?limit=2&marker=1234')
         for s in sl:
-            self.assertTrue(isinstance(s, servers.Server))
+            self.assertIsInstance(s, servers.Server)
 
     def test_get_server_details(self):
         s = cs.servers.get(1234)
         cs.assert_called('GET', '/servers/1234')
-        self.assertTrue(isinstance(s, servers.Server))
+        self.assertIsInstance(s, servers.Server)
         self.assertEqual(s.id, 1234)
         self.assertEqual(s.status, 'BUILD')
 
     def test_get_server_promote_details(self):
         s1 = cs.servers.list(detailed=False)[0]
         s2 = cs.servers.list(detailed=True)[0]
-        self.assertNotEquals(s1._info, s2._info)
+        self.assertNotEqual(s1._info, s2._info)
         s1.get()
         self.assertEqual(s1._info, s2._info)
 
@@ -58,7 +70,7 @@ class ServersTest(utils.TestCase):
             }
         )
         cs.assert_called('POST', '/servers')
-        self.assertTrue(isinstance(s, servers.Server))
+        self.assertIsInstance(s, servers.Server)
 
     def test_create_server_boot_from_volume_with_nics(self):
         old_boot = cs.servers._boot
@@ -88,7 +100,7 @@ class ServersTest(utils.TestCase):
                 nics=nics
             )
             cs.assert_called('POST', '/os-volumes_boot')
-            self.assertTrue(isinstance(s, servers.Server))
+            self.assertIsInstance(s, servers.Server)
 
         test_create_server_from_volume()
 
@@ -105,7 +117,7 @@ class ServersTest(utils.TestCase):
             },
         )
         cs.assert_called('POST', '/servers')
-        self.assertTrue(isinstance(s, servers.Server))
+        self.assertIsInstance(s, servers.Server)
 
     def test_create_server_userdata_unicode(self):
         s = cs.servers.create(
@@ -121,7 +133,7 @@ class ServersTest(utils.TestCase):
             },
         )
         cs.assert_called('POST', '/servers')
-        self.assertTrue(isinstance(s, servers.Server))
+        self.assertIsInstance(s, servers.Server)
 
     def test_create_server_userdata_utf8(self):
         s = cs.servers.create(
@@ -137,7 +149,7 @@ class ServersTest(utils.TestCase):
             },
         )
         cs.assert_called('POST', '/servers')
-        self.assertTrue(isinstance(s, servers.Server))
+        self.assertIsInstance(s, servers.Server)
 
     def _create_disk_config(self, disk_config):
         s = cs.servers.create(
@@ -147,7 +159,7 @@ class ServersTest(utils.TestCase):
             disk_config=disk_config
         )
         cs.assert_called('POST', '/servers')
-        self.assertTrue(isinstance(s, servers.Server))
+        self.assertIsInstance(s, servers.Server)
 
         # verify disk config param was used in the request:
         last_request = cs.client.callstack[-1]
@@ -196,6 +208,11 @@ class ServersTest(utils.TestCase):
         reval = cs.assert_called('POST', '/servers/1234/metadata',
                          {'metadata': {'test_key': 'test_value'}})
 
+    def test_set_server_meta_item(self):
+        s = cs.servers.set_meta_item(1234, 'test_key', 'test_value')
+        reval = cs.assert_called('PUT', '/servers/1234/metadata/test_key',
+                         {'meta': {'test_key': 'test_value'}})
+
     def test_find(self):
         server = cs.servers.find(name='sample-server')
         cs.assert_called('GET', '/servers', pos=-2)
@@ -249,6 +266,15 @@ class ServersTest(utils.TestCase):
     def test_rebuild_server_disk_config_manual(self):
         self._rebuild_resize_disk_config('MANUAL')
 
+    def test_rebuild_server_preserve_ephemeral(self):
+        s = cs.servers.get(1234)
+        s.rebuild(image=1, preserve_ephemeral=True)
+        cs.assert_called('POST', '/servers/1234/action')
+        body = cs.client.callstack[-1][-1]
+        d = body['rebuild']
+        self.assertIn('preserve_ephemeral', d)
+        self.assertEqual(d['preserve_ephemeral'], True)
+
     def test_resize_server(self):
         s = cs.servers.get(1234)
         s.resize(flavor=1)
@@ -420,9 +446,35 @@ class ServersTest(utils.TestCase):
         self.assertEqual(cs.servers.get_console_output(s, length=50), success)
         cs.assert_called('POST', '/servers/1234/action')
 
+    # Testing password methods with the following password and key
+    #
+    # Clear password: FooBar123
+    #
+    # RSA Private Key: novaclient/tests/idfake.pem
+    #
+    # Encrypted password
+    # OIuEuQttO8Rk93BcKlwHQsziDAnkAm/V6V8VPToA8ZeUaUBWwS0gwo2K6Y61Z96r
+    # qG447iRz0uTEEYq3RAYJk1mh3mMIRVl27t8MtIecR5ggVVbz1S9AwXJQypDKl0ho
+    # QFvhCBcMWPohyGewDJOhDbtuN1IoFI9G55ZvFwCm5y7m7B2aVcoLeIsJZE4PLsIw
+    # /y5a6Z3/AoJZYGG7IH5WN88UROU3B9JZGFB2qtPLQTOvDMZLUhoPRIJeHiVSlo1N
+    # tI2/++UsXVg3ow6ItqCJGgdNuGG5JB+bslDHWPxROpesEIHdczk46HCpHQN8f1sk
+    # Hi/fmZZNQQqj1Ijq0caOIw==
+
     def test_get_password(self):
         s = cs.servers.get(1234)
-        self.assertEqual(s.get_password('/foo/id_rsa'), '')
+        self.assertEqual(s.get_password('novaclient/tests/idfake.pem'),
+                         b'FooBar123')
+        cs.assert_called('GET', '/servers/1234/os-server-password')
+
+    def test_get_password_without_key(self):
+        s = cs.servers.get(1234)
+        self.assertEqual(s.get_password(),
+            'OIuEuQttO8Rk93BcKlwHQsziDAnkAm/V6V8VPToA8ZeUaUBWwS0gwo2K6Y61Z96r'
+            'qG447iRz0uTEEYq3RAYJk1mh3mMIRVl27t8MtIecR5ggVVbz1S9AwXJQypDKl0ho'
+            'QFvhCBcMWPohyGewDJOhDbtuN1IoFI9G55ZvFwCm5y7m7B2aVcoLeIsJZE4PLsIw'
+            '/y5a6Z3/AoJZYGG7IH5WN88UROU3B9JZGFB2qtPLQTOvDMZLUhoPRIJeHiVSlo1N'
+            'tI2/++UsXVg3ow6ItqCJGgdNuGG5JB+bslDHWPxROpesEIHdczk46HCpHQN8f1sk'
+            'Hi/fmZZNQQqj1Ijq0caOIw==')
         cs.assert_called('GET', '/servers/1234/os-server-password')
 
     def test_clear_password(self):
@@ -458,6 +510,14 @@ class ServersTest(utils.TestCase):
         cs.servers.get_spice_console(s, 'fake')
         cs.assert_called('POST', '/servers/1234/action')
 
+    def test_get_rdp_console(self):
+        s = cs.servers.get(1234)
+        s.get_rdp_console('fake')
+        cs.assert_called('POST', '/servers/1234/action')
+
+        cs.servers.get_rdp_console(s, 'fake')
+        cs.assert_called('POST', '/servers/1234/action')
+
     def test_create_image(self):
         s = cs.servers.get(1234)
         s.create_image('123')
