@@ -1,4 +1,17 @@
-import os
+#
+#    Licensed under the Apache License, Version 2.0 (the "License"); you may
+#    not use this file except in compliance with the License. You may obtain
+#    a copy of the License at
+#
+#         http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#    License for the specific language governing permissions and limitations
+#    under the License.
+
+import json
 import pkg_resources
 import re
 import sys
@@ -9,56 +22,16 @@ import prettytable
 import six
 
 from novaclient import exceptions
+from novaclient.openstack.common import cliutils
+from novaclient.openstack.common.gettextutils import _
+from novaclient.openstack.common import jsonutils
 from novaclient.openstack.common import strutils
 
 
-def arg(*args, **kwargs):
-    """Decorator for CLI args."""
-    def _decorator(func):
-        add_arg(func, *args, **kwargs)
-        return func
-    return _decorator
+arg = cliutils.arg
+env = cliutils.env
 
-
-def env(*args, **kwargs):
-    """
-    returns the first environment variable set
-    if none are non-empty, defaults to '' or keyword arg default
-    """
-    for arg in args:
-        value = os.environ.get(arg, None)
-        if value:
-            return value
-    return kwargs.get('default', '')
-
-
-def add_arg(f, *args, **kwargs):
-    """Bind CLI arguments to a shell.py `do_foo` function."""
-
-    if not hasattr(f, 'arguments'):
-        f.arguments = []
-
-    # NOTE(sirp): avoid dups that can occur when the module is shared across
-    # tests.
-    if (args, kwargs) not in f.arguments:
-        # Because of the sematics of decorator composition if we just append
-        # to the options list positional options will appear to be backwards.
-        f.arguments.insert(0, (args, kwargs))
-
-
-def bool_from_str(val):
-    """Convert a string representation of a bool into a bool value."""
-
-    if not val:
-        return False
-    try:
-        return bool(int(val))
-    except ValueError:
-        if val.lower() in ['true', 'yes', 'y']:
-            return True
-        if val.lower() in ['false', 'no', 'n']:
-            return False
-        raise
+VALID_KEY_REGEX = re.compile(r"[\w\.\- :]+$", re.UNICODE)
 
 
 def add_resource_manager_extra_kwargs_hook(f, hook):
@@ -87,8 +60,8 @@ def get_resource_manager_extra_kwargs(f, args, allow_conflicts=False):
 
         conflicting_keys = set(hook_kwargs.keys()) & set(extra_kwargs.keys())
         if conflicting_keys and not allow_conflicts:
-            raise Exception("Hook '%(hook_name)s' is attempting to redefine"
-                    " attributes '%(conflicting_keys)s'" %
+            raise Exception(_("Hook '%(hook_name)s' is attempting to redefine"
+                    " attributes '%(conflicting_keys)s'") %
                     {'hook_name': hook_name,
                         'conflicting_keys': conflicting_keys})
 
@@ -97,27 +70,6 @@ def get_resource_manager_extra_kwargs(f, args, allow_conflicts=False):
     return extra_kwargs
 
 
-def unauthenticated(f):
-    """
-    Adds 'unauthenticated' attribute to decorated function.
-    Usage:
-        @unauthenticated
-        def mymethod(f):
-            ...
-    """
-    f.unauthenticated = True
-    return f
-
-
-def isunauthenticated(f):
-    """
-    Checks to see if the function is marked as not requiring authentication
-    with the @unauthenticated decorator. Returns True if decorator is
-    set to True, False otherwise.
-    """
-    return getattr(f, 'unauthenticated', False)
-
-
 def service_type(stype):
     """
     Adds 'service_type' attribute to decorated function.
@@ -143,6 +95,11 @@ def pretty_choice_list(l):
     return ', '.join("'%s'" % i for i in l)
 
 
+def pretty_choice_dict(d):
+    """Returns a formatted dict as 'key=value'."""
+    return pretty_choice_list(['%s=%s' % (k, d[k]) for k in sorted(d.keys())])
+
+
 def print_list(objs, fields, formatters={}, sortby_index=None):
     if sortby_index is None:
         sortby = None
@@ -163,22 +120,67 @@ def print_list(objs, fields, formatters={}, sortby_index=None):
                 else:
                     field_name = field.lower().replace(' ', '_')
                 data = getattr(o, field_name, '')
+                if data is None:
+                    data = '-'
                 row.append(data)
         pt.add_row(row)
 
     if sortby is not None:
-        print(strutils.safe_encode(pt.get_string(sortby=sortby)))
+        result = strutils.safe_encode(pt.get_string(sortby=sortby))
     else:
-        print(strutils.safe_encode(pt.get_string()))
+        result = strutils.safe_encode(pt.get_string())
+
+    print(result)
+
+
+def _flatten(data, prefix=None):
+    """Flatten a dict, using name as a prefix for the keys of dict.
+
+    >>> _flatten('cpu_info', {'arch':'x86_64'})
+    [('cpu_info_arch': 'x86_64')]
+
+    """
+    if isinstance(data, dict):
+        for key, value in six.iteritems(data):
+            new_key = '%s_%s' % (prefix, key) if prefix else key
+            if isinstance(value, (dict, list)):
+                for item in _flatten(value, new_key):
+                    yield item
+            else:
+                yield new_key, value
+    else:
+        yield prefix, data
+
+
+def flatten_dict(data):
+    """Return a new dict whose sub-dicts have been merged into the
+    original.  Each of the parents keys are prepended to the child's
+    to prevent collisions.  Any string elements will be JSON parsed
+    before flattening.
+
+    >>> flatten_dict({'service': {'host':'cloud9@compute-068', 'id': 143}})
+    {'service_host': colud9@compute-068', 'service_id': 143}
+
+    """
+    data = data.copy()
+    # Try and decode any nested JSON structures.
+    for key, value in six.iteritems(data):
+        if isinstance(value, six.string_types):
+            try:
+                data[key] = json.loads(value)
+            except ValueError:
+                pass
+
+    return dict(_flatten(data))
 
 
 def print_dict(d, dict_property="Property", dict_value="Value", wrap=0):
     pt = prettytable.PrettyTable([dict_property, dict_value], caching=False)
     pt.align = 'l'
-    for k, v in six.iteritems(d):
+    for k, v in sorted(d.items()):
         # convert dict to str to check length
-        if isinstance(v, dict):
-            v = str(v)
+        if isinstance(v, (dict, list)):
+            v = jsonutils.dumps(v)
         if wrap > 0:
             v = textwrap.fill(str(v), wrap)
         # if value has a newline, add in multiple rows
@@ -190,13 +192,25 @@ def print_dict(d, dict_property="Property", dict_value="Value", wrap=0):
                 pt.add_row([col1, line])
                 col1 = ''
         else:
+            if v is None:
+                v = '-'
             pt.add_row([k, v])
-    print(strutils.safe_encode(pt.get_string()))
+
+    result = strutils.safe_encode(pt.get_string())
+
+    print(result)
 
 
 def find_resource(manager, name_or_id, **find_args):
     """Helper for the _find_* methods."""
-    # first try to get entity as integer id
+    # for str id which is not uuid (for Flavor search currently)
+    if getattr(manager, 'is_alphanum_id_allowed', False):
+        try:
+            return manager.get(name_or_id)
+        except exceptions.NotFound:
+            pass
+
+    # try to get entity as integer id
     try:
         return manager.get(int(name_or_id))
     except (TypeError, ValueError, exceptions.NotFound):
@@ -204,18 +218,12 @@ def find_resource(manager, name_or_id, **find_args):
 
     # now try to get entity as uuid
     try:
-        uuid.UUID(strutils.safe_encode(name_or_id))
-        return manager.get(name_or_id)
+        tmp_id = strutils.safe_encode(name_or_id)
+        uuid.UUID(tmp_id)
+        return manager.get(tmp_id)
     except (TypeError, ValueError, exceptions.NotFound):
         pass
 
-    # for str id which is not uuid (for Flavor search currently)
-    if getattr(manager, 'is_alphanum_id_allowed', False):
-        try:
-            return manager.get(name_or_id)
-        except exceptions.NotFound:
-            pass
-
     try:
         try:
             return manager.find(human_id=name_or_id, **find_args)
@@ -230,13 +238,15 @@ def find_resource(manager, name_or_id, **find_args):
             kwargs.update(find_args)
             return manager.find(**kwargs)
         except exceptions.NotFound:
-            msg = "No %s with a name or ID of '%s' exists." % \
-                (manager.resource_class.__name__.lower(), name_or_id)
+            msg = _("No %(class)s with a name or ID of '%(name)s' exists.") % \
+                  {'class': manager.resource_class.__name__.lower(),
+                   'name': name_or_id}
             raise exceptions.CommandError(msg)
     except exceptions.NoUniqueMatch:
-        msg = ("Multiple %s matches found for '%s', use an ID to be more"
-               " specific." % (manager.resource_class.__name__.lower(),
-                               name_or_id))
+        msg = (_("Multiple %(class)s matches found for '%(name)s', use an ID "
+                 "to be more specific.") %
+               {'class': manager.resource_class.__name__.lower(),
+                'name': name_or_id})
         raise exceptions.CommandError(msg)
 
 
@@ -328,27 +338,6 @@ def import_class(import_str):
     __import__(mod_str)
     return getattr(sys.modules[mod_str], class_str)
 
-_slugify_strip_re = re.compile(r'[^\w\s-]')
-_slugify_hyphenate_re = re.compile(r'[-\s]+')
-
-
-# http://code.activestate.com/recipes/
-#   577257-slugify-make-a-string-usable-in-a-url-or-filename/
-def slugify(value):
-    """
-    Normalizes string, converts to lowercase, removes non-alpha characters,
-    and converts spaces to hyphens.
-
-    From Django's "django/template/defaultfilters.py".
-    """
-    import unicodedata
-    if not isinstance(value, six.text_type):
-        value = six.text_type(value)
-    value = unicodedata.normalize('NFKD', value).encode('ascii',
-                    'ignore').decode("ascii")
-    value = six.text_type(_slugify_strip_re.sub('', value).strip().lower())
-    return _slugify_hyphenate_re.sub('-', value)
-
 
 def _load_entry_point(ep_name, name=None):
     """Try to load the entry point ep_name that matches name."""
@@ -366,3 +355,13 @@ def is_integer_like(val):
         return True
     except (TypeError, ValueError, AttributeError):
         return False
+
+
+def validate_flavor_metadata_keys(keys):
+    for key in keys:
+        valid_name = VALID_KEY_REGEX.match(key)
+        if not valid_name:
+            msg = _('Invalid key: "%s". Keys may only contain letters, '
+                    'numbers, spaces, underscores, periods, colons and '
+                    'hyphens.')
+            raise exceptions.CommandError(msg % key)
