@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 # Copyright 2012 OpenStack Foundation
 # Copyright 2013 IBM Corp.
 #
@@ -16,16 +15,17 @@
 
 from novaclient import client
 from novaclient.v1_1 import agents
-from novaclient.v1_1 import certs
-from novaclient.v1_1 import cloudpipe
 from novaclient.v1_1 import aggregates
 from novaclient.v1_1 import availability_zones
-from novaclient.v1_1 import coverage_ext
-from novaclient.v1_1 import flavors
+from novaclient.v1_1 import certs
+from novaclient.v1_1 import cloudpipe
+from novaclient.v1_1 import fixed_ips
 from novaclient.v1_1 import flavor_access
+from novaclient.v1_1 import flavors
 from novaclient.v1_1 import floating_ip_dns
-from novaclient.v1_1 import floating_ips
 from novaclient.v1_1 import floating_ip_pools
+from novaclient.v1_1 import floating_ips
+from novaclient.v1_1 import floating_ips_bulk
 from novaclient.v1_1 import fping
 from novaclient.v1_1 import hosts
 from novaclient.v1_1 import hypervisors
@@ -38,14 +38,12 @@ from novaclient.v1_1 import quotas
 from novaclient.v1_1 import security_group_rules
 from novaclient.v1_1 import security_groups
 from novaclient.v1_1 import servers
+from novaclient.v1_1 import services
 from novaclient.v1_1 import usage
 from novaclient.v1_1 import virtual_interfaces
-from novaclient.v1_1 import volumes
 from novaclient.v1_1 import volume_snapshots
 from novaclient.v1_1 import volume_types
-from novaclient.v1_1 import services
-from novaclient.v1_1 import fixed_ips
-from novaclient.v1_1 import floating_ips_bulk
+from novaclient.v1_1 import volumes
 
 
 class Client(object):
@@ -74,7 +72,7 @@ class Client(object):
                   volume_service_name=None, timings=False,
                   bypass_url=None, os_cache=False, no_cache=True,
                   http_log_debug=False, auth_system='keystone',
-                  auth_plugin=None,
+                  auth_plugin=None, auth_token=None,
                   cacert=None, tenant_id=None):
         # FIXME(comstud): Rename the api_key argument above when we
         # know it's not being used as keyword argument
@@ -116,7 +114,6 @@ class Client(object):
         self.fixed_ips = fixed_ips.FixedIPsManager(self)
         self.floating_ips_bulk = floating_ips_bulk.FloatingIPBulkManager(self)
         self.os_cache = os_cache or not no_cache
-        self.coverage = coverage_ext.CoverageManager(self)
         self.availability_zones = \
             availability_zones.AvailabilityZoneManager(self)
 
@@ -132,6 +129,7 @@ class Client(object):
                                     projectid=project_id,
                                     tenant_id=tenant_id,
                                     auth_url=auth_url,
+                                    auth_token=auth_token,
                                     insecure=insecure,
                                     timeout=timeout,
                                     auth_system=auth_system,
