@@ -13,8 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from six.moves.urllib import parse
+
 from novaclient import base
-from novaclient.openstack.common.py3kcompat import urlutils
 
 
 def _quote_domain(domain):
@@ -24,7 +25,7 @@ def _quote_domain(domain):
     but Routes tends to choke on them, so we need an extra level of
     by-hand quoting here.
     """
-    return urlutils.quote(domain.replace('.', '%2E'))
+    return parse.quote(domain.replace('.', '%2E'))
 
 
 class FloatingIPDNSDomain(base.Resource):
@@ -101,7 +102,7 @@ class FloatingIPDNSEntryManager(base.Manager):
     def get_for_ip(self, domain, ip):
         """Return a list of entries for the given domain and ip or name."""
         qparams = {'ip': ip}
-        params = "?%s" % urlutils.urlencode(qparams)
+        params = "?%s" % parse.urlencode(qparams)
 
         return self._list("/os-floating-ip-dns/%s/entries%s" %
                               (_quote_domain(domain), params),
