@@ -18,9 +18,9 @@ Security group interface (1.1 extension).
 """
 
 import six
+from six.moves.urllib import parse
 
 from novaclient import base
-from novaclient.openstack.common.py3kcompat import urlutils
 
 
 class SecurityGroup(base.Resource):
@@ -90,7 +90,7 @@ class SecurityGroupManager(base.ManagerWithFind):
 
         qparams = dict((k, v) for (k, v) in six.iteritems(search_opts) if v)
 
-        query_string = '?%s' % urlutils.urlencode(qparams) if qparams else ''
+        query_string = '?%s' % parse.urlencode(qparams) if qparams else ''
 
         return self._list('/os-security-groups%s' % query_string,
                           'security_groups')
