@@ -18,9 +18,9 @@ Volume interface (1.1 extension).
 """
 
 import six
+from six.moves.urllib import parse
 
 from novaclient import base
-from novaclient.openstack.common.py3kcompat import urlutils
 
 
 class Volume(base.Resource):
@@ -89,7 +89,7 @@ class VolumeManager(base.ManagerWithFind):
 
         qparams = dict((k, v) for (k, v) in six.iteritems(search_opts) if v)
 
-        query_string = '?%s' % urlutils.urlencode(qparams) if qparams else ''
+        query_string = '?%s' % parse.urlencode(qparams) if qparams else ''
 
         if detailed is True:
             return self._list("/volumes/detail%s" % query_string, "volumes")
